import { gql, useMutation } from "@apollo/client";
import { Button, LinearProgress, makeStyles, Paper } from "@material-ui/core";
import { Form, Formik, Field, FormikHelpers } from "formik";
import { TextField } from "formik-material-ui";
import { not } from "ramda";
import React, { FC } from "react";
import {
  CreateProjectInput,
  UpdateProjectInput,
  Project,
} from "../generated/graphql";
import * as Yup from "yup";
import { useRouter } from "@curi/react-dom";

type Values = Partial<Project>;

interface Props {
  project: Values;
}

const useStyles = makeStyles({
  root: {
    overflow: "hidden",
  },
  form: {
    margin: 16,
  },
});

const CREATE_PROJECT = gql`
  mutation CreateProject($input: CreateProjectInput!) {
    createProject(project: $input) {
      id
      name
      comment
      webUrl
      webHookSecret
      sshUrl
    }
  }
`;

const UPDATE_PROJECT = gql`
  mutation UpdateProject($input: UpdateProjectInput!) {
    updateProject(project: $input) {
      id
      name
      comment
      webUrl
      webHookSecret
      sshUrl
    }
  }
`;

export const ProjectEditor: FC<Props> = ({ project }) => {
  const isCreate = not("id" in project);

  const [createProject] = useMutation<{ createProject: Project }>(
    CREATE_PROJECT,
    {
      update(cache, { data }) {
        cache.modify({
          fields: {
            findProjects(exitingProjects = []) {
              const newProjectRef = cache.writeFragment({
                data: data!.createProject,
                fragment: gql`
                  fragment newProject on Project {
                    id
                    name
                    comment
                    webUrl
                    sshUrl
                    webHookSecret
                  }
                `,
              });
              return [newProjectRef, ...exitingProjects];
            },
          },
        });
      },
    }
  );
  const [updateProject] = useMutation(UPDATE_PROJECT);

  const router = useRouter();
  const submitForm = async (
    values: Values,
    formikHelpers: FormikHelpers<Values>
  ) => {
    try {
      let projectId: string | undefined = project.id;
      if (isCreate) {
        await createProject({
          variables: {
            input: values,
          },
        }).then(({ data }) => (projectId = data!.createProject.id));
      } else {
        await updateProject({
          variables: {
            id: (project as Project).id,
            input: values,
          },
        });
      }
      router.navigate({
        url: router.url({
          name: "project-detail",
          params: {
            projectId,
          },
        }),
        method: "replace",
      });
    } finally {
      formikHelpers.setSubmitting(false);
    }
  };

  const classes = useStyles();
  return (
    <Paper className={classes.root}>
      <Formik
        initialValues={project}
        validationSchema={Yup.object({
          name: Yup.string()
            .max(32, "Must be 32 characters or less")
            .required("Required"),
          comment: Yup.string()
            .max(32, "Must be 32 characters or less")
            .required("Required"),
          webUrl: Yup.string()
            .matches(
              /^(https?:\/\/)?([\da-z.-]+\.[a-z.]{2,6}|[\d.]+)([\\/:?=&#]{1}[\da-z.-]+)*[/\\?]?$/i,
              "Enter correct url!"
            )
            .max(256, "Must be 256 characters or less")
            .required("Required"),
          sshUrl: Yup.string()
            .matches(
              /^(?:ssh:\/\/)?(?:[\w\d-_]+@)?(?:[\w\d-_]+\.)*\w{2,10}(?::\d{1,5})?(?:\/[\w\d-_.]+)*$/,
              "Enter correct url!"
            )
            .max(256, "Must be 256 characters or less")
            .required("Required"),
          webHookSecret: Yup.string()
            .max(16, "Must be 16 characters or less")
            .required("Required"),
        })}
        onSubmit={submitForm}
      >
        {({ submitForm, isSubmitting }) => (
          <Form className={classes.form}>
            <Field component={TextField} name="name" label="Name" fullWidth />
            <Field
              component={TextField}
              name="comment"
              label="Comment"
              fullWidth
              margin="normal"
            />
            <Field
              component={TextField}
              name="webUrl"
              label="Project URL"
              placeholder="The project website url"
              fullWidth
              margin="normal"
            />
            <Field
              component={TextField}
              name="sshUrl"
              label="Git SSH URL"
              placeholder="The Git remote SSH URL"
              fullWidth
              margin="normal"
            />
            <Field
              component={TextField}
              name="webHookSecret"
              label="Webhook Secret"
              fullWidth
              margin="normal"
            />
            {isSubmitting && <LinearProgress />}
            <Button
              variant="contained"
              color="primary"
              disabled={isSubmitting}
              onClick={submitForm}
              fullWidth
            >
              Submit
            </Button>
          </Form>
        )}
      </Formik>
    </Paper>
  );
};
